from django.core.management.base import BaseCommand, CommandError
from django.contrib.auth.models import User, Group
from opensubmit.signalhandlers import check_permission_system

class Command(BaseCommand):
    help = 'Makes sure that the OpenSubmit permission system is up and valid'
    def handle(self, *args, **options):
        print("Scanning existing users ...")
        check_permission_system()



import logging
logger = logging.getLogger('OpenSubmit')

class CourseRegister():
	'''
		Allows to enable a course for an user based on a GET parameter with the course ID.
		Sanity checks are done in the model method for adding it, which may throw a 404 exception.
	'''
	def process_request(self, request):
	    if 'course' in request.GET and request.user and request.user.is_authenticated():
        	course_id=request.GET['course']

        	request.user.profile.add_course_safe(course_id)
        	logger.debug("Adding user %u to course %s"%(request.user.pk, course_id))
		return None



from django import template
from django.template.defaultfilters import stringfilter
from opensubmit.models import Submission
import os
import opensubmit

register = template.Library()

@register.filter(name='basename')
@stringfilter
def basename(value):
    return os.path.basename(value)

@register.filter(name='state_label_css')
def state_label_css(subm):
    green_label = "badge label label-success"
    red_label = "badge label label-important"
    grey_label = "badge label label-info"
    # We expect a submission as input
    if subm.is_closed() and subm.grading:
        if subm.grading.means_passed:
            return green_label
        else:
            return red_label
    if subm.state in [subm.SUBMITTED_TESTED, subm.SUBMITTED, subm.TEST_FULL_PENDING, subm.GRADED, subm.TEST_FULL_FAILED]:
        return green_label
    if subm.state in [subm.TEST_COMPILE_FAILED, subm.TEST_VALIDITY_FAILED]:
        return red_label
    return grey_label

@register.assignment_tag
def setting(name):
    return getattr(opensubmit.settings, name, "")

@register.inclusion_tag('inclusion_tags/details_table.html')
def details_table(submission):
    return {'submission': submission}




from django.contrib.admin import ModelAdmin, StackedInline
from django.db.models import Q
from opensubmit.models import Submission

def submissions(submfile):
    while submfile.replaced_by is not None:
        submfile = submfile.replaced_by
    subms = submfile.submissions.all()
    return ','.join([str(sub) for sub in subms])


def not_withdrawn(submfile):
    return submfile.replaced_by is None
not_withdrawn.boolean = True

class SubmissionFileAdmin(ModelAdmin):
    list_display = ['__unicode__', 'fetched', submissions, not_withdrawn]

    class Media:
        css = {'all': ('css/admin.css',)}

    def get_queryset(self, request):
        ''' Restrict the listed submission files for the current user.'''
        qs = super(SubmissionFileAdmin, self).get_queryset(request)
        if request.user.is_superuser:
            return qs
        else:
            return qs.filter(Q(submissions__assignment__course__tutors__pk=request.user.pk) | Q(submissions__assignment__course__owner=request.user)).distinct()



from django.conf.urls import patterns, include, url
from django.conf.urls.static import static
from django.contrib.staticfiles.urls import staticfiles_urlpatterns
from django.contrib.auth.decorators import login_required
from django.http import HttpResponseRedirect

from opensubmit import views, admin, api

urlpatterns = [
    url(r'^$', views.index, name='index'),
    url(r'^logout/$', views.logout, name='logout'),
    url(r'^dashboard/$', views.dashboard, name='dashboard'),
    url(r'^details/(?P<subm_id>\d+)/$', views.details, name='details'),
    url(r'^assignments/(?P<ass_id>\d+)/new/$', views.new, name='new'),
    url(r'^assignments/(?P<ass_id>\d+)/perftable/$', views.perftable, name='perftable'),
    url(r'^assignments/(?P<ass_id>\d+)/duplicates/$', views.duplicates, name='duplicates'),
    url(r'^withdraw/(?P<subm_id>\d+)/$', views.withdraw, name='withdraw'),
    url(r'^preview/(?P<subm_id>\d+)/$', views.preview, name='preview'),
    url(r'^update/(?P<subm_id>\d+)/$', views.update, name='update'),
    url(r'^course/(?P<course_id>\d+)/gradingtable/$', views.gradingtable, name='gradingtable'),
    url(r'^course/(?P<course_id>\d+)/archive/$', views.coursearchive, name='coursearchive'),
    url(r'^course/(?P<course_id>\d+)/mail2all/$', views.mail2all, name='mail2all'),
    url(r'^jobs/$', api.jobs, name='jobs'),
    url(r'^download/(?P<obj_id>\d+)/(?P<filetype>\w+)/secret=(?P<secret>\w+)$', api.download, name='download_secret'),
    url(r'^download/(?P<obj_id>\d+)/(?P<filetype>\w+)/$', api.download, name='download'),
    url(r'^machine/(?P<machine_id>\d+)/$', views.machine, name='machine'),
    url(r'^machines/$', api.machines, name='machines'),
    url(r'^settings/$', views.settings, name='settings'),
    url(r'^courses/$', views.courses, name='courses'),
    url(r'^mergeusers/$', views.mergeusers, name='mergeusers'),

    url('', include('social.apps.django_app.urls', namespace='social')),
    url(r'^lti/$', views.lti, name='lti'),
    url(r'^teacher/opensubmit/$', lambda x: HttpResponseRedirect('/teacher/')),
    url(r'^teacher/', include(admin.teacher_backend.urls)),
    url(r'^admin/opensubmit/$', lambda x: HttpResponseRedirect('/admin/')),
    url(r'^admin/', include(admin.admin_backend.urls)),
    url(r'^grappelli/', include('grappelli.urls')), # grappelli URLS
]

# only working when DEBUG==FALSE
# on production systems, static files are served directly by Apache
urlpatterns += staticfiles_urlpatterns()

import urls
def show_urls(urllist, depth=0):
    for entry in urllist:
        print "  " * depth, entry.regex.pattern
        if hasattr(entry, 'url_patterns'):
            show_urls(entry.url_patterns, depth + 1)
#show_urls(urls.urlpatterns)


# Grading scheme admin interface

from django.contrib.admin import ModelAdmin
from django.db.models import Q
from django.core.urlresolvers import resolve
from opensubmit.models import Course, Grading

def gradings(gradingScheme):
    ''' Determine the list of gradings in this scheme as rendered string.
        TODO: Use nice little icons instead of (p) / (f) marking.
    '''
    result = []
    for grading in gradingScheme.gradings.all():
        if grading.means_passed:
            result.append(str(grading) + " (pass)")
        else:
            result.append(str(grading) + " (fail)")
    return '  -  '.join(result)


def courses(gradingScheme):
    # determine the courses that use this grading scheme in one of their assignments
    course_ids = gradingScheme.assignments.all().values_list('course', flat=True)
    courses = Course.objects.filter(pk__in=course_ids)
    return ",\n".join([str(course) for course in courses])


class GradingSchemeAdmin(ModelAdmin):
    list_display = ['__unicode__', gradings, courses]

    class Media:
        css = {'all': ('css/admin.css',)}


    def formfield_for_dbfield(self, db_field, **kwargs):
        ''' Offer only gradings that are not used by other schemes, which means they are used by this scheme or not at all.'''
        if db_field.name == "gradings":
            request=kwargs['request']
            try:
                #TODO:  MockRequst object from unit test does not contain path information, so an exception occurs during test.
                #       Find a test-suite compatible solution here.
                obj=resolve(request.path).args[0]
                filterexpr=Q(schemes=obj) | Q(schemes=None)
                kwargs['queryset'] = Grading.objects.filter(filterexpr).distinct()
            except:
                pass
        return super(GradingSchemeAdmin, self).formfield_for_dbfield(db_field, **kwargs)



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.db import models, migrations


class Migration(migrations.Migration):

    dependencies = [
        ('opensubmit', '0006_testmachine_address'),
    ]

    operations = [
        migrations.AlterField(
            model_name='testmachine',
            name='address',
            field=models.CharField(help_text=b'Internal IP address of the test machine, at the time of registration.', max_length=50, null=True),
        ),
        migrations.AlterField(
            model_name='testmachine',
            name='host',
            field=models.CharField(help_text=b'UUID of the test machine, independent from IP address.', max_length=50, null=True),
        ),
    ]



from django.db import models
from django.utils import timezone
from django.conf import settings
from django.core.urlresolvers import reverse

from .submission import Submission
from .submissiontestresult import SubmissionTestResult

class Assignment(models.Model):
    '''
        An assignment for which students can submit their solution.
    '''

    title = models.CharField(max_length=200)
    course = models.ForeignKey('Course', related_name='assignments')
    download = models.URLField(max_length=200, verbose_name="Link for assignment description")
    created = models.DateTimeField(auto_now_add=True, editable=False)
    gradingScheme = models.ForeignKey('GradingScheme', related_name="assignments", verbose_name="grading scheme")
    publish_at = models.DateTimeField(default=timezone.now)
    soft_deadline = models.DateTimeField(blank=True, null=True, help_text="Deadline shown to students. After this point in time, submissions are still possible. Leave empty for only using a hard deadline.")
    hard_deadline = models.DateTimeField(help_text="Deadline after which submissions are no longer possible.")
    has_attachment = models.BooleanField(default=False, verbose_name="Student file upload ?", help_text="Activate this if the students must upload a (document / ZIP /TGZ) file as solution. Otherwise, they can only fill the notes field.")
    attachment_test_timeout = models.IntegerField(default=30, verbose_name="Timout for tests", help_text="Timeout (in seconds) after which the compilation / validation test / full test is cancelled. The submission is marked as invalid in this case. Intended for student code with deadlocks.")
    attachment_test_compile = models.BooleanField(default=False, verbose_name="Compile test ?", help_text="If activated, the student upload is uncompressed and 'make' is executed on one of the test machines.")
    attachment_test_validity = models.FileField(upload_to="testscripts", blank=True, null=True, verbose_name='Validation script', help_text="If given, the student upload is uncompressed, compiled and the script is executed for it on a test machine. Student submissions are marked as valid if this script was successful.")
    validity_script_download = models.BooleanField(default=False, verbose_name='Download of validation script ?', help_text='If activated, the students can download the validation script for offline analysis.')
    attachment_test_full = models.FileField(upload_to="testscripts", blank=True, null=True, verbose_name='Full test script', help_text='Same as the validation script, but executed AFTER the hard deadline to determine final grading criterias for the submission. Results are not shown to students.')
    test_machines = models.ManyToManyField('TestMachine', blank=True, related_name="assignments", help_text="The test machines that will take care of submissions for this assignment.")

    class Meta:
        app_label = 'opensubmit'

    def gradable_submissions(self):
        qs = self.valid_submissions()
        qs = qs.filter(state__in=[Submission.GRADING_IN_PROGRESS, Submission.SUBMITTED_TESTED, Submission.TEST_FULL_FAILED, Submission.SUBMITTED])
        return qs

    def grading_unfinished_submissions(self):
        qs = self.valid_submissions()
        qs = qs.filter(state__in=[Submission.GRADING_IN_PROGRESS])
        return qs

    def graded_submissions(self):
        qs = self.valid_submissions().filter(state__in=[Submission.GRADED])
        return qs

    def grading_url(self):
        '''
            Determines the teacher backend link to the filtered list of gradable submissions for this assignment.
        '''
        grading_url="%s?coursefilter=%u&assignmentfilter=%u&statefilter=tobegraded"%(
                            reverse('teacher:opensubmit_submission_changelist'),
                            self.course.pk, self.pk
                        )
        return grading_url

    def authors(self):
        qs = self.valid_submissions().values_list('authors',flat=True).distinct()
        return qs

    def valid_submissions(self):
        qs = self.submissions.exclude(state=Submission.WITHDRAWN)
        return qs

    def uploads(self):
        '''
            Return a queryset for all non-replaced file uploads for  this assignment.
        '''
        from .submissionfile import SubmissionFile
        return SubmissionFile.valid_ones.filter(submissions__assignment=self)

    def uploads_by_md5(self):
        '''
           Return uploads, but ordered by MD5 sum. Crucial for duplicate view regroup to work properly.
        '''
        return self.uploads().order_by('md5')

    def has_perf_results(self):
        '''
            Figure out if any submission for this assignment has performance data being available.
        '''
        num_results = SubmissionTestResult.objects.filter(perf_data__isnull=False).filter(submission_file__submissions__assignment=self).count()
        return num_results != 0

    def validity_test_url(self):
        '''
            Return absolute download URL for validity test script.
            Using reverse() seems to be broken with FORCE_SCRIPT in use, so we use direct URL formulation.
        '''
        if self.pk and self.has_validity_test():
            return settings.MAIN_URL + "/download/%u/validity_testscript/secret=%s" % (self.pk, settings.JOB_EXECUTOR_SECRET)
        else:
            return None

    def full_test_url(self):
        '''
            Return absolute download URL for full test script.
            Using reverse() seems to be broken with FORCE_SCRIPT in use, so we use direct URL formulation.
        '''
        if self.pk and self.has_full_test():
            return settings.MAIN_URL + "/download/%u/full_testscript/secret=%s" % (self.pk, settings.JOB_EXECUTOR_SECRET)
        else:
            return None

    def has_validity_test(self):
        return str(self.attachment_test_validity).strip() != ""

    def has_full_test(self):
        return str(self.attachment_test_full).strip() != ""

    def attachment_is_tested(self):
        return self.attachment_test_compile is True or self.has_validity_test() or self.has_full_test()

    def __unicode__(self):
        return unicode(self.title)

    def can_create_submission(self, user=None):
        '''
            Central access control for submitting things related to assignments.
        '''

        if user:
            # Super users, course owners and tutors should be able to test their validations
            # before the submission is officially possible.
            # They should also be able to submit after the deadline.
            if user.is_superuser or user is self.course.owner or self.course.tutors.filter(pk=user.pk).exists():
                return True
            if self.course not in user.profile.user_courses():
                # The user is not enrolled in this assignment's course.
                return False

            if user.authored.filter(assignment=self).exclude(state=Submission.WITHDRAWN).count() > 0:
                # User already has a valid submission for this assignment.
                return False

        if self.hard_deadline < timezone.now():
            # Hard deadline has been reached.
            return False

        if self.publish_at > timezone.now():
            # The assignment has not yet been published.
            return False

        return True

    def authors_valid(self, authors=()):
        for author in authors:
            if not self.can_create_submission(author):
                return False

        return True



from .course import Course
from .grading import Grading
from .gradingscheme import GradingScheme
from .testmachine import TestMachine
from .submission import Submission
from .submissionfile import SubmissionFile
from .submissiontestresult import SubmissionTestResult
from .assignment import Assignment
from .userprofile import UserProfile




from django.contrib.auth.models import User, Permission, Group
from opensubmit.models import Course, Grading, GradingScheme, Assignment, SubmissionFile, Submission, TestMachine
from django.contrib.admin.sites import AdminSite

from user import UserAdmin
from course import CourseAdmin
from grading import GradingAdmin
from gradingscheme import GradingSchemeAdmin
from assignment import AssignmentAdmin
from submissionfile import SubmissionFileAdmin
from submission import SubmissionAdmin

class AdminBackend(AdminSite):
	site_header = "Administrator Backend"
	pass

admin_backend = AdminBackend(name="admin")
admin_backend.register(User, UserAdmin)
admin_backend.register(Course, CourseAdmin)
admin_backend.register(TestMachine)
admin_backend.register(Permission)
admin_backend.register(Group)


class TeacherBackend(AdminSite):
	site_header = "Teacher Backend"
	pass

teacher_backend = TeacherBackend(name="teacher")
teacher_backend.register(Grading, GradingAdmin)
teacher_backend.register(GradingScheme, GradingSchemeAdmin)
teacher_backend.register(Assignment, AssignmentAdmin)
teacher_backend.register(SubmissionFile, SubmissionFileAdmin)
teacher_backend.register(Submission, SubmissionAdmin)
teacher_backend.register(Course, CourseAdmin)
teacher_backend.register(TestMachine)



from django.db.models.signals import post_save, pre_save
from django.dispatch import receiver
from django.contrib.auth.models import User, Group, Permission
from opensubmit.models import Submission, Course, SubmissionFile

STUDENT_TUTORS_GROUP_NAME = "Student Tutors"
COURSE_OWNERS_GROUP_NAME = "Course Owners"

def check_permission_system():
    '''
        This methods makes sure that:

        1.) All neccessary user groups exist.
        2.) All these groups have the right database permissions assigned.
        3.) Tutors have backend rights.
        4.) Course owners have backend rights.
        5.) Students have no backend rights.

        The permission objects were already generated by the Django database initialization,
        so we can assume them to be given.

        This method is idempotent and does not touch manually assigned permissions.
    '''
    tutor_perms = ( "change_submission", "delete_submission",
                    "change_submissionfile", "delete_submissionfile" )
    owner_perms = ( "add_assignment", "change_assignment", "delete_assignment",
                    "add_grading", "change_grading",  "delete_grading",
                    "add_gradingscheme", "change_gradingscheme", "delete_gradingscheme",
                    "add_submission", "change_submission", "delete_submission",
                    "add_submissionfile", "change_submissionfile", "delete_submissionfile",
                    "change_course" )

    # Give all tutor users staff rights and add them to the tutors permission group
    tutors = User.objects.filter(courses_tutoring__isnull=False)
    tutors.update(is_staff=True)
    # If the app crashes here, you may have duplicate group objects, which must be fixed manually in the DB.
    tutor_group, created = Group.objects.get_or_create(name=STUDENT_TUTORS_GROUP_NAME)
    # If the app crashes here, you may have duplicate permission objects, which must be fixed manually in the DB.
    tutor_group.permissions = [Permission.objects.get(codename=perm) for perm in tutor_perms]
    tutor_group.user_set.add(*tutors)
    tutor_group.save()

    # Give all course owner users staff rights and add them to the course owners permission group
    owners = User.objects.filter(courses__isnull=False)
    owners.update(is_staff=True)
    # If the app crashes here, you may have duplicate group objects, which must be fixed manually in the DB.
    owner_group, created = Group.objects.get_or_create(name=COURSE_OWNERS_GROUP_NAME)
    # If the app crashes here, you may have duplicate permission objects, which must be fixed manually in the DB.
    owner_group.permissions = [Permission.objects.get(codename=perm) for perm in owner_perms]
    owner_group.user_set.add(*owners)
    owner_group.save()

    # Make sure that pure students (no tutor, no course owner, no superuser) have no backend access at all
    pure_students = User.objects.filter(courses__isnull=True, courses_tutoring__isnull=True, is_superuser=False)
    pure_students.update(is_staff=False)

@receiver(post_save, sender=User)
def post_user_save(sender,instance, signal, created, **kwargs):
    """
        Make sure that all neccessary user groups exist and have the right permissions,
        directly after the auth system was installed. We detect this by waiting for the admin
        account creation here, which smells hacky.
        We need that automatism for the test database creation, people not calling the configure tool and similar cases.
    """
    if instance.is_staff and created:
        check_permission_system()

@receiver(post_save, sender=SubmissionFile)
def submissionfile_post_save(sender,instance, signal, created, **kwargs):
    '''
        Update MD5 field for newly uploaded files.
    '''
    if created:
        instance.md5 = instance.attachment_md5()
        instance.save()

@receiver(post_save, sender=Submission)
def submission_post_save(sender, instance, **kwargs):
    ''' Several sanity checks after we got a valid submission object.'''
    # Make the submitter an author
    if instance.submitter not in instance.authors.all():
        instance.authors.add(instance.submitter)
        instance.save()
    # Mark all existing submissions for this assignment by these authors as invalid.
    # This fixes a race condition with parallel new submissions in multiple browser windows by the same user.
    # Solving this as pre_save security exception does not work, since we have no instance with valid foreign keys to check there.
    # Considering that this runs also on tutor correction backend activities, it also serves as kind-of cleanup functionality
    # for multiplse submissions by the same students for the same assignment - however they got in here.
    if instance.state == instance.get_initial_state():
        for author in instance.authors.all():
            same_author_subm = User.objects.get(pk=author.pk).authored.all().exclude(pk=instance.pk).filter(assignment=instance.assignment)
            for subm in same_author_subm:
                subm.state = Submission.WITHDRAWN
                subm.save()

@receiver(post_save, sender=Course)
def course_post_save(sender, instance, **kwargs):
    '''
        After creating / modifying a course, make sure that only tutors and course owners have backend access rights.
        We do that here since tutor addition and removal ends up in a course modification signal.
    '''
    check_permission_system()


from django.db import models
from django.contrib.auth.models import User
from django.utils import timezone
from django.core.urlresolvers import reverse

from .assignment import Assignment
from .submission import Submission

class Course(models.Model):
    title = models.CharField(max_length=200)
    created = models.DateTimeField(auto_now_add=True, editable=False)
    owner = models.ForeignKey(User, related_name='courses', help_text="Only this user can change the course details and create new assignments.")
    tutors = models.ManyToManyField(User, blank=True, related_name='courses_tutoring', help_text="These users can edit / grade submissions for the course.")
    homepage = models.URLField(max_length=200, verbose_name="Course description link")
    active = models.BooleanField(default=True, help_text="Only assignments and submissions of active courses are shown to students and tutors. Use this flag for archiving past courses.")
    max_authors = models.PositiveSmallIntegerField(default=1, help_text="Maximum number of authors (= group size) for assignments in this course.")
    lti_key = models.CharField(max_length=100, null=True, blank=True, help_text="Key to be used by an LTI consumer when accessing this course.")
    lti_secret = models.CharField(max_length=100, null=True, blank=True, help_text="Secret to be used by an LTI consumer when accessing this course.")

    class Meta:
        app_label = 'opensubmit'

    def __unicode__(self):
        return unicode(self.title)

    def open_assignments(self):
        qs = Assignment.objects.filter(hard_deadline__gt=timezone.now())
        qs = qs.filter(publish_at__lt=timezone.now())
        qs = qs.filter(course=self)
        qs = qs.order_by('soft_deadline').order_by('hard_deadline').order_by('title')
        return qs

    def gradable_submissions(self):
        '''
            Queryset for the gradable submissions that are worth a look by tutors.
        '''
        qs = self._valid_submissions()
        qs = qs.filter(state__in=[Submission.GRADING_IN_PROGRESS, Submission.SUBMITTED_TESTED, Submission.TEST_FULL_FAILED, Submission.SUBMITTED])
        return qs

    def graded_submissions(self):
        '''
            Queryset for the graded submissions, which are worth closing.
        '''
        qs = self._valid_submissions().filter(state__in=[Submission.GRADED])
        return qs

    def grading_url(self):
        '''
            Determines the teacher backend link to the filtered list of gradable submissions for this course.
        '''
        grading_url="%s?coursefilter=%u&statefilter=tobegraded"%(
                            reverse('teacher:opensubmit_submission_changelist'),
                            self.pk
                        )
        return grading_url


    def authors(self):
        '''
            Queryset for all distinct authors this course had so far. Important for statistics.
            Note that this may be different from the list of people being registered for the course,
            f.e. when they submit something and the leave the course.
        '''
        qs = self._valid_submissions().values_list('authors',flat=True).distinct()
        return qs

    def _valid_submissions(self):
        qs = Submission.objects.filter(assignment__course=self).exclude(state=Submission.WITHDRAWN)
        return qs

def lti_secret(key):
    '''
        Consumer function as expected by blti package.
    '''
    try:
        return Course.objects.get(lti_key=key).lti_secret
    except:
        return None



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.db import models, migrations


class Migration(migrations.Migration):

    dependencies = [
        ('opensubmit', '0005_auto_20151007_2253'),
    ]

    operations = [
        migrations.AddField(
            model_name='testmachine',
            name='address',
            field=models.TextField(help_text=b'Internal IP address of the test machine at the time of registration.', null=True),
        ),
    ]



import opensubmit
from opensubmit import settings


def footer(request):
    return {
        'main_url': settings.MAIN_URL,
        'opensubmit_version': settings.VERSION,
        'opensubmit_admin': settings.ADMINS[0][1],
    }



'''
    This module contains administrative functionality that is available as command-line tool "opensubmit-web".
'''

import os, pwd, grp, urllib, sys, shutil
from ConfigParser import RawConfigParser
from pkg_resources import Requirement, resource_filename

def django_admin(args):
    '''
        Run something like it would be done through Django's manage.py.
    '''
    from django.core.management import execute_from_command_line
    from django.core.exceptions import ImproperlyConfigured
    os.environ.setdefault("DJANGO_SETTINGS_MODULE", "opensubmit.settings")
    try:
        execute_from_command_line([sys.argv[0]]+args)
    except ImproperlyConfigured as e:
        print(str(e))
        exit(-1)

def apache_config(config, outputfile):
    '''
        Generate a valid Apache configuration file, based on the given settings.
    '''
    if os.path.exists(outputfile):
        os.rename(outputfile, outputfile+".old")
        print "Renamed existing Apache config file to "+outputfile+".old"

    from opensubmit import settings
    f = open(outputfile,'w')
    print "Generating Apache configuration in "+outputfile
    subdir = (len(settings.HOST_DIR)>0)
    text = """
    # OpenSubmit Configuration for Apache 2.4
    # These directives are expected to live in some <VirtualHost> block
    """
    if subdir:
        text += "Alias /%s/static/ %s\n"%(settings.HOST_DIR, settings.STATIC_ROOT)
        text += "    WSGIScriptAlias /%s %s/wsgi.py\n"%(settings.HOST_DIR, settings.SCRIPT_ROOT)
    else:
        text += "Alias /static/ %s\n"%(settings.STATIC_ROOT)
        text += "    WSGIScriptAlias / %s/wsgi.py"%(settings.SCRIPT_ROOT)
    text += """
    WSGIPassAuthorization On
    <Directory {static_path}>
         Require all granted
    </Directory>
    <Directory {install_path}>
         <Files wsgi.py>
              Require all granted
         </Files>
    </Directory>
    """.format(static_path=settings.STATIC_ROOT, install_path=settings.SCRIPT_ROOT)

    f.write(text)
    f.close()

def check_path(directory):
    '''
        Checks if the directories for this path exist, and creates them in case.
    '''
    if directory != '':
        if not os.path.exists(directory):
            os.makedirs(directory, 0775)   # rwxrwxr-x

def check_file(filepath):
    '''
        - Checks if the parent directories for this path exist.
        - Checks that the file exists.
        - Donates the file to the web server user.

        TODO: This is Debian / Ubuntu specific.
    '''
    check_path(os.path.dirname(filepath))
    if not os.path.exists(filepath):
        print "WARNING: File does not exist. Creating it: %s"%filepath
        open(filepath, 'a').close()
    try:
        print "Setting access rights for %s for www-data user"%(filepath)
        uid = pwd.getpwnam("www-data").pw_uid
        gid = grp.getgrnam("www-data").gr_gid
        os.chown(filepath, uid, gid)
        os.chmod(filepath, 0660) # rw-rw---
    except:
        print "WARNING: Could not adjust file system permissions for %s. Make sure your web server can write into it."%filepath

def check_web_config_consistency(config):
    '''
        Check the web application config file for consistency.
    '''
    login_conf_deps = {
        'LOGIN_TWITTER': ['LOGIN_TWITTER_OAUTH_KEY', 'LOGIN_TWITTER_OAUTH_SECRET'],
        'LOGIN_GOOGLE': ['LOGIN_GOOGLE_OAUTH_KEY', 'LOGIN_GOOGLE_OAUTH_SECRET'],
        'LOGIN_GITHUB': ['LOGIN_GITHUB_OAUTH_KEY', 'LOGIN_GITHUB_OAUTH_SECRET']
    }

    print "Checking configuration of the OpenSubmit web application..."
    # Let Django's manage.py load the settings file, to see if this works in general
    django_admin(["check"])
    # Check configured host
    try:
        urllib.urlopen(config.get("server", "HOST"))
    except Exception as e:
        # This may be ok, when the admin is still setting up to server
        print "The configured HOST seems to be invalid at the moment: "+str(e)
    # Check configuration dependencies
    for k, v in login_conf_deps.iteritems():
        if config.getboolean('login', k):
            for needed in v:
                if len(config.get('login', needed)) < 1:
                    print "ERROR: You have enabled %s in settings.ini, but %s is not set."%(k, needed)
                    return False
    # Check media path
    check_path(config.get('server', 'MEDIA_ROOT'))
    # Prepare empty log file, in case the web server has no creation rights
    log_file = config.get('server', 'LOG_FILE')
    print "Preparing log file at "+log_file
    check_file(log_file)
    # If SQLite database, adjust file system permissions for the web server
    if config.get('database','DATABASE_ENGINE') == 'sqlite3':
        name = config.get('database','DATABASE_NAME')
        if not os.path.isabs(name):
            print "ERROR: Your SQLite database name must be an absolute path. The web server must have directory access permissions for this path."
            return False
        check_file(config.get('database','DATABASE_NAME'))
    # everything ok
    return True

def check_web_config(config_path):
    '''
        Try to load the Django settings.
        If this does not work, than settings file does not exist.
    '''
    WEB_CONFIG_FILE = config_path+'/settings.ini'
    WEB_TEMPLATE = "opensubmit/settings.ini.template"                   # relative to package path
    print "Looking for config files ..."
    config = RawConfigParser()
    try:
        config.readfp(open(WEB_CONFIG_FILE))
        print "Config file found at "+WEB_CONFIG_FILE
        return config
    except IOError:
        print "ERROR: Seems like the config file does not exist."
        print "       I am creating a new one. Please edit it and re-run this command."
        check_path(config_path)
        orig = resource_filename(Requirement.parse("opensubmit-web"),WEB_TEMPLATE)
        shutil.copy(orig, WEB_CONFIG_FILE)
        check_file(WEB_CONFIG_FILE)
        return None    # Manual editing is needed before further proceeding with the fresh file

def check_web_db():
    '''
        Everything related to database checks and updates.
    '''
    print "Testing for neccessary database migrations..."
    django_admin(["migrate"])             # apply schema migrations
    print "Checking the OpenSubmit permission system..."
    django_admin(["fixperms"])            # configure permission system, of needed
    return True

def console_script(fsroot='/'):
    '''
        The main entry point for the production administration script 'opensubmit-web', installed by setuptools.

        The argument allows the test suite to override the root of all paths used in here.
    '''

    if len(sys.argv) == 1:
        print "opensubmit-web [configure|createsuperuser|fixperms|fixchecksums|help]"
        exit(0)

    if "help" in sys.argv:
        print "configure:        Check config files and database for correct installation of the OpenSubmit web server."
        print "createsuperuser:  (Re-)Creates the superuser account for the OpenSubmit installation."
        print "fixperms:         Check and fix student and tutor permissions"
        print "fixchecksums:     Re-create all student file checksums (for duplicate detection)"
        print "help:             Print this help"
        return 0

    if "configure" in sys.argv:
        config = check_web_config(fsroot+'etc/opensubmit')
        if not config:
            return          # Let them first fix the config file before trying a DB access
        if not check_web_config_consistency(config):
            return
        if not check_web_db():
            return
        print("Preparing static files for web server...")
        django_admin(["collectstatic","--noinput","--clear"])
        apache_config(config, fsroot+'etc/opensubmit/apache24.conf')

    if "createsuperuser" in sys.argv:
        django_admin(["createsuperuser"])

    if "fixperms" in sys.argv:
        django_admin(["fixperms"])

    if "fixchecksums" in sys.argv:
        django_admin(["fixchecksums"])



from django.contrib.admin import ModelAdmin

def grading_schemes(grading):
    ''' The list of grading schemes using this grading.'''
    return ",\n".join([str(scheme) for scheme in grading.schemes.all()])


def means_passed(grading):
    return grading.means_passed
means_passed.boolean = True

class GradingAdmin(ModelAdmin):
    list_display = ['__unicode__', grading_schemes, means_passed]



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.db import models, migrations
from opensubmit.models.submissionfile import upload_path
import django.utils.timezone
from django.conf import settings


class Migration(migrations.Migration):

    dependencies = [
        migrations.swappable_dependency(settings.AUTH_USER_MODEL),
    ]

    operations = [
        migrations.CreateModel(
            name='Assignment',
            fields=[
                ('id', models.AutoField(verbose_name='ID', serialize=False, auto_created=True, primary_key=True)),
                ('title', models.CharField(max_length=200)),
                ('download', models.URLField()),
                ('created', models.DateTimeField(auto_now_add=True)),
                ('publish_at', models.DateTimeField(default=django.utils.timezone.now)),
                ('soft_deadline', models.DateTimeField(null=True, blank=True)),
                ('hard_deadline', models.DateTimeField()),
                ('has_attachment', models.BooleanField(default=False)),
                ('attachment_test_timeout', models.IntegerField(default=30)),
                ('attachment_test_compile', models.BooleanField(default=False)),
                ('attachment_test_validity', models.FileField(null=True, upload_to=b'testscripts', blank=True)),
                ('validity_script_download', models.BooleanField(default=False)),
                ('attachment_test_full', models.FileField(null=True, upload_to=b'testscripts', blank=True)),
            ],
            options={
            },
            bases=(models.Model,),
        ),
        migrations.CreateModel(
            name='Course',
            fields=[
                ('id', models.AutoField(verbose_name='ID', serialize=False, auto_created=True, primary_key=True)),
                ('title', models.CharField(max_length=200)),
                ('created', models.DateTimeField(auto_now_add=True)),
                ('homepage', models.URLField()),
                ('active', models.BooleanField(default=True)),
                ('max_authors', models.PositiveSmallIntegerField(default=1)),
                ('owner', models.ForeignKey(related_name=b'courses', to=settings.AUTH_USER_MODEL)),
                ('tutors', models.ManyToManyField(related_name=b'courses_tutoring', null=True, to=settings.AUTH_USER_MODEL, blank=True)),
            ],
            options={
            },
            bases=(models.Model,),
        ),
        migrations.CreateModel(
            name='Grading',
            fields=[
                ('id', models.AutoField(verbose_name='ID', serialize=False, auto_created=True, primary_key=True)),
                ('title', models.CharField(max_length=20)),
                ('means_passed', models.BooleanField(default=True)),
            ],
            options={
            },
            bases=(models.Model,),
        ),
        migrations.CreateModel(
            name='GradingScheme',
            fields=[
                ('id', models.AutoField(verbose_name='ID', serialize=False, auto_created=True, primary_key=True)),
                ('title', models.CharField(max_length=200)),
                ('gradings', models.ManyToManyField(related_name=b'schemes', to='opensubmit.Grading')),
            ],
            options={
            },
            bases=(models.Model,),
        ),
        migrations.CreateModel(
            name='Submission',
            fields=[
                ('id', models.AutoField(verbose_name='ID', serialize=False, auto_created=True, primary_key=True)),
                ('notes', models.TextField(max_length=200, blank=True)),
                ('created', models.DateTimeField(auto_now_add=True)),
                ('modified', models.DateTimeField(auto_now=True, null=True)),
                ('grading_notes', models.TextField(max_length=1000, null=True, blank=True)),
                ('grading_file', models.FileField(null=True, upload_to=upload_path, blank=True)),
                ('state', models.CharField(default=b'R', max_length=2, choices=[(b'R', b'Received'), (b'W', b'Withdrawn'), (b'S', b'Submitted'), (b'PC', b'Compilation test pending'), (b'FC', b'Compilation test failed'), (b'PV', b'Validity test pending'), (b'FV', b'Validity test failed'), (b'PF', b'Full test pending'), (b'FF', b'All but full test passed, grading pending'), (b'ST', b'All tests passed, grading pending'), (b'GP', b'Grading not finished'), (b'G', b'Grading finished'), (b'C', b'Closed, student notified'), (b'CT', b'Closed, full test pending')])),
                ('assignment', models.ForeignKey(related_name=b'submissions', to='opensubmit.Assignment')),
                ('authors', models.ManyToManyField(help_text=b'', related_name=b'authored', to=settings.AUTH_USER_MODEL)),
            ],
            options={
            },
            bases=(models.Model,),
        ),
        migrations.CreateModel(
            name='SubmissionFile',
            fields=[
                ('id', models.AutoField(verbose_name='ID', serialize=False, auto_created=True, primary_key=True)),
                ('attachment', models.FileField(upload_to=upload_path)),
                ('fetched', models.DateTimeField(null=True, editable=False)),
                ('replaced_by', models.ForeignKey(blank=True, to='opensubmit.SubmissionFile', null=True)),
            ],
            options={
            },
            bases=(models.Model,),
        ),
        migrations.CreateModel(
            name='SubmissionTestResult',
            fields=[
                ('id', models.AutoField(verbose_name='ID', serialize=False, auto_created=True, primary_key=True)),
                ('created', models.DateTimeField(auto_now_add=True)),
                ('result', models.TextField(null=True, blank=True)),
                ('kind', models.CharField(max_length=2, choices=[(b'c', b'Compilation test'), (b'v', b'Validation test'), (b'f', b'Full test')])),
                ('perf_data', models.TextField(null=True, blank=True)),
            ],
            options={
            },
            bases=(models.Model,),
        ),
        migrations.CreateModel(
            name='TestMachine',
            fields=[
                ('id', models.AutoField(verbose_name='ID', serialize=False, auto_created=True, primary_key=True)),
                ('host', models.TextField(null=True)),
                ('last_contact', models.DateTimeField(editable=False)),
                ('config', models.TextField(null=True)),
            ],
            options={
            },
            bases=(models.Model,),
        ),
        migrations.CreateModel(
            name='UserProfile',
            fields=[
                ('id', models.AutoField(verbose_name='ID', serialize=False, auto_created=True, primary_key=True)),
                ('courses', models.ManyToManyField(related_name=b'participants', null=True, to='opensubmit.Course', blank=True)),
                ('user', models.OneToOneField(to=settings.AUTH_USER_MODEL)),
            ],
            options={
            },
            bases=(models.Model,),
        ),
        migrations.AddField(
            model_name='submissiontestresult',
            name='machine',
            field=models.ForeignKey(related_name=b'test_results', to='opensubmit.TestMachine'),
            preserve_default=True,
        ),
        migrations.AddField(
            model_name='submissiontestresult',
            name='submission_file',
            field=models.ForeignKey(related_name=b'test_results', to='opensubmit.SubmissionFile'),
            preserve_default=True,
        ),
        migrations.AddField(
            model_name='submission',
            name='file_upload',
            field=models.ForeignKey(related_name=b'submissions', blank=True, to='opensubmit.SubmissionFile', null=True),
            preserve_default=True,
        ),
        migrations.AddField(
            model_name='submission',
            name='grading',
            field=models.ForeignKey(blank=True, to='opensubmit.Grading', null=True),
            preserve_default=True,
        ),
        migrations.AddField(
            model_name='submission',
            name='submitter',
            field=models.ForeignKey(related_name=b'submitted', to=settings.AUTH_USER_MODEL),
            preserve_default=True,
        ),
        migrations.AddField(
            model_name='assignment',
            name='course',
            field=models.ForeignKey(related_name=b'assignments', to='opensubmit.Course'),
            preserve_default=True,
        ),
        migrations.AddField(
            model_name='assignment',
            name='gradingScheme',
            field=models.ForeignKey(related_name=b'assignments', to='opensubmit.GradingScheme'),
            preserve_default=True,
        ),
        migrations.AddField(
            model_name='assignment',
            name='test_machines',
            field=models.ManyToManyField(to='opensubmit.TestMachine', null=True, blank=True),
            preserve_default=True,
        ),
    ]



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.db import models, migrations
from django.conf import settings


class Migration(migrations.Migration):

    dependencies = [
        ('opensubmit', '0003_auto_20150831_1900'),
    ]

    operations = [
        migrations.AddField(
            model_name='userprofile',
            name='student_id',
            field=models.CharField(max_length=30, null=True),
        ),
        migrations.AlterField(
            model_name='userprofile',
            name='user',
            field=models.OneToOneField(related_name='profile', to=settings.AUTH_USER_MODEL),
        ),
    ]



from django.utils.translation import ugettext_lazy as _
from grappelli.dashboard import modules, Dashboard
from django.core.urlresolvers import reverse

from opensubmit import settings
from opensubmit.models import TestMachine, Submission

class TeacherDashboard(Dashboard):

    class Media:
        css = {'all': ('css/admin.css',)}

    def init_with_context(self, context):
        # Put database models in  column
        self.children.append(modules.ModelList(
            title='Database Management',
            column=2,
            exclude=('django.contrib.*','opensubmit.models.grading.*'),
        ))

        # Put course action boxes in column
        try:
            courses = context.request.user.profile.tutor_courses().all()
        except:
            courses = []

        for course in courses:
            # Prepare course-related links
            links=[]
            links.append(['Check all submissions',course.grading_url(), False])
            ass_url="%s?course__id__exact=%u"%(
                                reverse('teacher:opensubmit_assignment_changelist'),
                                course.pk
                            )
            links.append(['Check assignments',ass_url, False])
            links.append(['Show current grading table',reverse('gradingtable', args=[course.pk]), False])
            links.append(['eMail to students',reverse('mail2all', args=[course.pk]), False])
            links.append(['Edit course',reverse('teacher:opensubmit_course_change', args=[course.pk]), False])
            links.append(['Download course archive',reverse('coursearchive', args=[course.pk]), False])

            # Add course group box to dashboard
            self.children.append(modules.Group(
                title=course.title,
                column=1,
                children=[
                    modules.LinkList(title="Actions",children=(links)),
                    modules.DashboardModule(title="Info",pre_content=
                        'Course URL for students: %s/?course=%u<br/>' % (settings.MAIN_URL, course.pk) +
                        "Open assignments: %u<br/>" % course.open_assignments().count() +
                        "Submissions to be graded: %u<br/>" % course.gradable_submissions().count() +
                        "Grading finished, not notified: %u<br/>" % course.graded_submissions().count() +
                        "Registered students: %u<br/>" % course.participants.count() +
                        "Authoring students: %u<br/>" % course.authors().count()
                    )
                ]
            ))

        # Put recent actions in third column
        self.children.append(modules.RecentActions(
            title='Recent teacher activities',
            column=3,
        ))


class AdminDashboard(Dashboard):

    class Media:
        css = {'all': ('css/admin.css',)}

    def init_with_context(self, context):
        # Put database models in  column
        self.children.append(modules.ModelList(
            title='Database Management',
            column=2,
        ))







"""
A Python Social backend base class that uses web server environment variables for authentication.
This is intended to perform the same as Django's RemoteUser middleware, but with all
the nice Python Social add-ons in the pipeline.

This backend class expects the following configuration variables being set by a derived class:

ENV_USERNAME   - The name of the environment variable containing the authenticated user name (mandatory).
ENV_EMAIL      - The name of the environment variable containing the eMail address of the authenticated user.
ENV_FIRST_NAME - The name of the environment variable containing the First name of the authenticated user.
ENV_LAST_NAME  - The name of the environment variable containing the Last name of the authenticated user.

It also expects auth_url to be implemented by the derived class.

Note: If you are using Apache + mod_wsgi, make sure to set 'WSGIPassAuthorization On'.
"""

from social.backends.base import BaseAuth
from social.exceptions import AuthMissingParameter
import os, logging

logger = logging.getLogger('OpenSubmit')

class ServerEnvAuth(BaseAuth):
    ENV_USERNAME = None
    ENV_EMAIL = None
    ENV_FIRST_NAME = None
    ENV_LAST_NAME = None

    def auth_url(self):
        """Must return redirect URL to auth provider."""
        raise NotImplementedError()

    def auth_complete(self, *args, **kwargs):
        """Completes loging process, must return user instance"""
        if self.ENV_USERNAME in os.environ:
            response = os.environ
        elif type(self.strategy).__name__ == "DjangoStrategy" and self.ENV_USERNAME in self.strategy.request.META:
            # Looks like the Django strategy. In this case, it might by mod_wsgi, which stores
            # authentication environment variables in request.META
            response = self.strategy.request.META
	else:
            raise AuthMissingParameter(self, "%s, found only: %s"%(self.ENV_USERNAME, str(os.environ)))
        kwargs.update({'response': response, 'backend': self})
        return self.strategy.authenticate(*args, **kwargs)

    def get_user_details(self, response):
        """ Complete with additional information from environment, as available. """
        result = {
            'username': response[self.ENV_USERNAME],
            'email': response.get(self.ENV_EMAIL, None),
            'first_name': response.get(self.ENV_FIRST_NAME, None),
            'last_name': response.get(self.ENV_LAST_NAME, None)
        }
        if result['first_name'] and result['last_name']:
            result['fullname']=result['first_name']+' '+result['last_name']
        logger.debug("Returning user details: "+str(result))
        return result

    def get_user_id(self, details, response):
        """Return a unique ID for the current user, by default from server response."""
        return response[self.ENV_USERNAME]




#!/usr/bin/env python
# -*- coding: utf-8 -*-

import os, sys, shutil

from setuptools import setup, find_packages
from distutils.command.clean import clean as _clean

with open('requirements.txt') as f:
    required = f.read().splitlines()

setup(
    name = 'opensubmit-web',
    version = open('VERSION').read(),
    url = 'https://github.com/troeger/opensubmit',
    license='AGPL',
    author = 'Peter Tröger',
    description = 'A web application for managing student assignment solutions in a university environment.',
    author_email = 'peter@troeger.eu',
    classifiers=[
        #   4 - Beta
        #   5 - Production/Stable
        'Development Status :: 4 - Beta',
        'Programming Language :: Python :: 2',
        'Programming Language :: Python :: 2.7'
    ],
    install_requires=required,
    packages = find_packages(exclude=['opensubmit.tests']),     # Just add Python packages
    include_package_data = True,                                # Consider MANIFEST.in
    entry_points={
        'console_scripts': [
            'opensubmit-web = opensubmit.cmdline:console_script',
        ],
    }
)





from django.db import models

class SubmissionTestResult(models.Model):
    '''
        An executor test result for a given submission file.
    '''

    COMPILE_TEST = 'c'
    VALIDITY_TEST = 'v'
    FULL_TEST = 'f'
    JOB_TYPES = (
        (COMPILE_TEST, 'Compilation test'),
        (VALIDITY_TEST, 'Validation test'),
        (FULL_TEST, 'Full test')
    )
    submission_file = models.ForeignKey('SubmissionFile', related_name="test_results")
    machine = models.ForeignKey('TestMachine', related_name="test_results")
    created = models.DateTimeField(auto_now_add=True, editable=False)
    result = models.TextField(null=True, blank=True)
    kind = models.CharField(max_length=2, choices=JOB_TYPES)
    perf_data = models.TextField(null=True, blank=True)

    class Meta:
        app_label = 'opensubmit'



from django.db import models, transaction
from django.contrib.auth.models import User
from django.utils import timezone
from django.shortcuts import get_object_or_404

from .assignment import Assignment
from .course import Course
from .submission import Submission

class UserProfile(models.Model):
    user = models.OneToOneField(User, related_name='profile')
    student_id = models.CharField(max_length=30, blank=True, null=True)
    courses = models.ManyToManyField(Course, blank=True, related_name='participants', limit_choices_to={'active__exact': True})

    class Meta:
        app_label = 'opensubmit'

    def add_course_safe(self, id):
        '''
            Adds a course for the user after conducting a set of sanity checks.
        '''
        course = get_object_or_404(Course, pk=int(id), active=True)
        if course not in self.courses.all():
            self.courses.add(course)

    def tutor_courses(self):
        '''
            Returns the list of courses this user is tutor or owner for.
        '''
        tutoring = self.user.courses_tutoring.all().filter(active__exact=True)
        owning = self.user.courses.all().filter(active__exact=True)
        result = (tutoring | owning).distinct()
        return result

    def user_courses(self):
        '''
            Returns the list of courses this user is subscribed for, or owning, or tutoring.
            This leads to the fact that tutors and owners don't need course membership.
        '''
        registered=self.courses.filter(active__exact=True).distinct()
        return (self.tutor_courses() | registered).distinct()

    def open_assignments(self):
        '''
            Returns the list of open assignments from the viewpoint of this user.
        '''
        qs = Assignment.objects.filter(hard_deadline__gt=timezone.now())
        qs = qs.filter(publish_at__lt=timezone.now())
        qs = qs.filter(course__in=self.user_courses())
        qs = qs.order_by('soft_deadline').order_by('hard_deadline').order_by('title')
        waiting_for_action = [subm.assignment for subm in self.user.authored.all().exclude(state=Submission.WITHDRAWN)]
        return [ass for ass in qs if ass not in waiting_for_action]

def db_fixes(user):
    '''
        Fix users that already exist and never got a user profile attached.
        This may be user accounts that were created by the Django Social or manually by the admin.

        TODO: This belongs into a User post_save handler.
    '''
    profile, created = UserProfile.objects.get_or_create(user=user)

def user_unicode(self):
    '''
        Monkey patch for getting better user name stringification,
        user proxies did not make the job.
        Django's custom user model feature would have needed to be introduced
        before the first syncdb, which does not work for existing installations.
'''
    if self.email:
        shortened = self.email.split('@')[0]
        return u'%s %s (%s@...)' % (self.first_name, self.last_name, shortened)
    elif self.first_name or self.last_name:
        return u'%s %s' % (self.first_name, self.last_name)
    elif self.username:
        return u'%s' % (self.username)
    else:
        return u'User %u' % (self.pk)
User.__unicode__ = user_unicode

@transaction.atomic
def move_user_data(primary, secondary):
    '''
        Moves all submissions and other data linked to the secondary user into the primary user.
        Nothing is deleted here, we just modify foreign user keys.
    '''
    # Update all submission authorships of the secondary to the primary
    submissions = Submission.objects.filter(authors__id=secondary.pk)
    for subm in submissions:
        if subm.submitter == secondary:
            subm.submitter = primary;
        subm.authors.remove(secondary)
        subm.authors.add(primary)
        subm.save()
    # Transfer course registrations
    try:
        for course in secondary.profile.courses.all():
            primary.profile.courses.add(course)
            primary.profile.save()
    except UserProfile.DoesNotExist:
        # That's a database consistency problem, but he will go away anyway
        pass
